@@ -21,13 +21,16 @@ using Gtk;
 using Gdk;
 using Glade;
 using Mono.Unix;
+using GnomeRDP;
 
 namespace GnomeRDP
 {
 	public class AboutDialog : GladeDialog
 	{
+#pragma warning disable 0649
 		[Widget]Label 		lbVersion;
 		[Widget]Gtk.Image 	iLogo;
+#pragma warning restore 0649
 		
 		public AboutDialog() : base("dlgAbout")
 		{
