@@ -26,11 +26,14 @@ using Mono.Data.SqliteClient;
 using System.Threading;
 using Mono.Unix;
 using System.Collections;
+using System.Reflection;
+using System.IO;
 
 namespace GnomeRDP 
 {
 	public class MainApp
-	{
+	{		
+#pragma warning disable 0649
 		[Widget]Gtk.Window			wndMain;
 		[Widget]ScrolledWindow 		scrolledwindow1;
 		[Widget]ToolButton			tbNew;
@@ -40,44 +43,48 @@ namespace GnomeRDP
 		[Widget]ToolButton			tbConnect;
 		[Widget]MenuBar				mainMenu;
 		[Widget]Statusbar			sbStatus;
-		private Gtk.Action 		actionQuit;
-		private Gtk.Action 		actionNew;
-		private Gtk.Action 		actionProperties;
-		private Gtk.Action 		actionDelete;
-		private Gtk.Action 		actionConnect;
-		private Gtk.Action			actionNewCategory;
-		private Gtk.Action			actionAbout;
-		private string 			HOME = Environment.GetEnvironmentVariable("HOME");
-		private string				par = "";
-		private ProcessCaller		prc;
-		private SessionTreeStore	sessionTreeStore;
-		private SessionTreeView	sessionTreeView;
-		private Pixbuf				icon;
-		private bool				notstarthidden = true;
+#pragma warning restore 0649
 		
-		public static void Main (string[] args)
+		private Gtk.Action actionQuit;
+		private Gtk.Action actionNew;
+		private Gtk.Action actionProperties;
+		private Gtk.Action actionDelete;
+		private Gtk.Action actionConnect;
+		private Gtk.Action actionNewCategory;
+		private Gtk.Action actionAbout;
+		private static string tmpFileName = System.IO.Path.GetTempFileName();
+		private string par = "";
+		private ProcessCaller prc;
+		private SessionTreeStore sessionTreeStore;
+		private SessionTreeView	sessionTreeView;		
+		private bool startVisible = true; 
+		private StatusIcon trayIcon;
+		
+		public static void Main(string[] args)
 		{
-			new MainApp (args);
+			new MainApp(args);
 		}	
 
-		public MainApp (string[] args) 
+		public MainApp(string[] args) 
 		{
-			Catalog.Init("gnome-rdp", GnomeRDP.Defines.LOCALE_DIR);
+			Catalog.Init("gnome-rdp", Defines.LOCALE_DIR);
+ 
 			if (this.ParseArguments(args))
 			{	
 				Gdk.Threads.Init();
 				Application.Init ();
 	
-				Glade.XML gxml = new Glade.XML (null, "gui.glade", "wndMain", "gnome-rdp");
+				Stream glade = Assembly.GetExecutingAssembly().GetManifestResourceStream("gui.glade");
+				Glade.XML gxml = new Glade.XML(glade, "wndMain", "gnome-rdp");
 				gxml.Autoconnect (this);
-				wndMain.Visible = notstarthidden;
 	
 				this.CreateActions();
-				this.CreateMenu();
 				if (this.InitializeComponent())
 				{
+					this.CreateMenu();
 					CreatePopupTrayMenu();
 					Gdk.Threads.Enter();
+					wndMain.Visible = startVisible;
 					Application.Run ();
 					Gdk.Threads.Leave();
 				}
@@ -93,7 +100,7 @@ namespace GnomeRDP
 				{
 					case "--start-hidden":
 					case "-m":
-						notstarthidden = false;
+						startVisible = false;
 						break;
 					case "--version":
 					case "-v":
@@ -105,6 +112,10 @@ namespace GnomeRDP
 						PrintOutHelp();
 						continueRunning = false;
 						break;
+					case "--debug":
+					case "-d":
+						Defines.DEBUG = true;
+						break;
 				}
 			}
 			return continueRunning;
@@ -113,7 +124,7 @@ namespace GnomeRDP
 		private void PrintOutVersion()
 		{
 			Console.WriteLine(Defines.PACKAGE + " " + Defines.VERSION);
-			Console.WriteLine("Copyright (c) 2005-2006 Balazs Varkonyi");
+			Console.WriteLine("Copyright (c) 2005-2008 Balazs Varkonyi");
 		}
 
 		private void PrintOutHelp()
@@ -130,33 +141,30 @@ namespace GnomeRDP
 		
 		private void CreatePopupTrayMenu()
 		{
-			this.icon = Gdk.Pixbuf.LoadFromResource("gnome-rdp-icon.png"); 
-			EventBox eb = new EventBox();
-			eb.Add(new Gtk.Image(icon.ScaleSimple(24, 24, Gdk.InterpType.Hyper)));
-			eb.ButtonPressEvent += new ButtonPressEventHandler(OnTrayClicked);
-			
-			TrayIcon Icon = new TrayIcon("Gnome-RDP");
-			Icon.Add(eb);
-			Icon.ShowAll();
+			this.trayIcon = new StatusIcon(Gdk.Pixbuf.LoadFromResource("gnome-rdp-icon.png")); 
+            this.trayIcon.Visible = true;
+            this.trayIcon.Activate += OnTrayIconClicked;
+            this.trayIcon.PopupMenu += OnTrayIconPopup;
+			this.trayIcon.Tooltip = Catalog.GetString("gnome-rdp");			
 		}
 		
 		private bool InitializeComponent()
 		{
-			Configuration cfg = new Configuration();
 			bool result = true;
 			
-			wndMain.Icon = Gdk.Pixbuf.LoadFromResource("gnome-rdp-icon.png"); 
+			wndMain.Icon = Gdk.Pixbuf.LoadFromResource("gnome-rdp-icon.png");
 			wndMain.DeleteEvent += new Gtk.DeleteEventHandler(OnWindowDeleteEvent);
 			wndMain.Title = "Gnome-RDP";
 			wndMain.Resizable = true;
 
 			sessionTreeStore = new SessionTreeStore();
 			sessionTreeView = new SessionTreeView(sessionTreeStore, false);
-			if (!System.IO.File.Exists(HOME + "/.gnome-rdp.db"))
+			if (!System.IO.File.Exists(Defines.DatabasePath))
 			{
-				cfg.CreateDatabase();
+				Options.CheckDatabase();
+				Configuration.CreateDatabase();
 			}
-			if (!cfg.CheckDatabaseVersion())
+			if (!Configuration.CheckDatabaseVersion())
 			{
 				MessageDialog md = new MessageDialog(this.wndMain, 
 					DialogFlags.Modal, MessageType.Error, ButtonsType.Close, 
@@ -165,6 +173,8 @@ namespace GnomeRDP
 				md.Destroy();
 				return false;
 			}
+			//ApplicationOptions.VerifyDatabase();
+			
 			this.RefreshSessionList();
 			scrolledwindow1.Add(sessionTreeView);
 			sessionTreeView.ExpandAll();
@@ -221,6 +231,15 @@ namespace GnomeRDP
 			categoryItem.Submenu = categoryMenu;
 			categoryMenu.Append(actionNewCategory.CreateMenuItem());
 			
+			//Options
+			Gtk.MenuItem optionsItem = new Gtk.MenuItem(Catalog.GetString("_Options"));
+			Gtk.Menu optionsSubMenu = new Gtk.Menu();
+			optionsItem.Submenu = optionsSubMenu;
+			Gtk.CheckMenuItem optionsUseKeyringItem = new CheckMenuItem(Catalog.GetString("_Use Keyring"));
+			optionsUseKeyringItem.Active = Options.UseKeyring;
+			optionsUseKeyringItem.Activated += OnUseKeyringClicked;
+			optionsSubMenu.Append(optionsUseKeyringItem);
+			
 			//Help
 			Gtk.Menu helpMenu = new Gtk.Menu();
 			Gtk.MenuItem helpItem = new Gtk.MenuItem(Catalog.GetString("_Help"));
@@ -230,6 +249,7 @@ namespace GnomeRDP
 			mainMenu.Append(fileItem);
 			mainMenu.Append(sessionItem);
 			mainMenu.Append(categoryItem);
+			mainMenu.Append(optionsItem);
 			mainMenu.Append(helpItem);
 			mainMenu.ShowAll();
 		}
@@ -242,10 +262,7 @@ namespace GnomeRDP
 
 		private void RefreshSessionListRecursive(int parentId, TreeIter parentIter)
 		{
-			ArrayList cfgList;
-			Configuration cf = new Configuration();
-			cfgList = cf.LoadSessions();
-			foreach (Configuration cfg in cfgList)
+			foreach (Configuration cfg in Configuration.LoadSessions())
 			{
 				if ((parentId == 0) && (cfg.ParentId == 0))
 				{
@@ -285,11 +302,27 @@ namespace GnomeRDP
 			}
 			if (config.Protocol == 1)
 			{
+//                VNC vnc = new VNC();
+                
+				if (config.Password.Length > 0) 
+				{
+                    prc = new ProcessCaller(
+                                            "vncpasswd",
+                                            tmpFileName,
+                                            "/",
+                                            true,
+                                            config.Password,
+                                            this.wndMain);
+                    prc.StartProcess();
+                    Thread.Sleep(1000);
+                }
+                
+                Console.WriteLine(par);
 				prc = new ProcessCaller(
-					"vncviewer", 
+					"xtightvncviewer", 
 					par, 
 					"/", 
-					true,
+					false,
 					config.Password, 
 					this.wndMain);
 				prc.StartProcess();
@@ -302,17 +335,8 @@ namespace GnomeRDP
 					"/",
 					true, 
 					config.Password, 
-					this.wndMain);
-				/*prc = new ProcessCaller(
-					"ssh",
-					"-e none -l root -p 22 192.168.100.1",
-					"/",
-					true, 
-					config.Password, 
-					this.wndMain);*/					
+					this.wndMain);					
 				prc.StartProcess();
-				/*TerminalDialog terminalDialog = new TerminalDialog();
-				terminalDialog.Run("ssh " + par);*/
 			}
 		}
 
@@ -370,7 +394,14 @@ namespace GnomeRDP
 				
 				if (config.User.Length > 0) 
 				{
-					par += " -u " + config.User;
+					if (config.User.Contains(" "))
+					{
+						par += string.Format(" -u \"{0}\"", config.User);
+					}
+					else
+					{
+						par += " -u " + config.User;
+					}
 				}
 				if (config.Password.Length > 0) 
 				{
@@ -420,26 +451,51 @@ namespace GnomeRDP
 			}
 			if (config.Protocol == 1)
 			{
-				if (config.User.Length > 0) 
+				par ="";
+				if (config.CompressionLevel == 1)
 				{
-					par += " -user " + config.User;
+					par += " -encodings \"zlib hextile raw\" "; 
+				}else if (config.CompressionLevel == 2)
+				{
+					par += " -encodings \"hextile zlib raw\" "; 
+				}else if (config.CompressionLevel == 3)
+				{
+					par += " -encodings \"raw zlib hextile\" "; 
 				}
-				if (config.Password.Length > 0) 
+				if (config.ImageQuality == 1)
 				{
-					par += " -autopass";
+					par += " -quality 2 ";
+				}else if (config.ImageQuality == 2)
+				{
+					par += " -quality 5 ";
+				}else if (config.ImageQuality == 3)
+				{
+					par += " -quality 9 ";
+				}else if (config.ImageQuality == 4)
+				{
+					par += " -truecolor ";
 				}
-				if (config.ConnectionType == 0)
+				if ((config.ImageQuality >0) || (config.CompressionLevel>0))
 				{
-					par += " -viewonly";
+					par += " -AutoSelect=0 ";
 				}
 				if (config.WindowMode > 0)
 				{
-					par += " -fullscreen";
+					par += " -fullscreen ";
 				}
-				par += " -xrm '*grabKeyboard: true'";
-				par += " -compresslevel " + config.CompressionLevel;
-				par += " -quality " + config.ImageQuality;
-				par += " -x11cursor " + config.Computer;
+				if (config.ConnectionType == 0)
+				{
+					par += " -viewonly ";
+				}else if (config.ConnectionType == 2)
+				{
+					par += " -shared";
+				}
+				if (config.Password.Length > 0) 
+				{
+					par += " -passwd ";
+					par += tmpFileName+" ";
+				}
+				par += config.Computer;
 			}
 			if (config.Protocol == 2)
 			{
@@ -478,9 +534,8 @@ namespace GnomeRDP
 					cfg = sessionTreeView.config;
 					break;
 				case "Gtk.ImageMenuItem":
-					ArrayList cfgList = cfg.LoadSessions();
 					ImageMenuItem item = (ImageMenuItem)sender;
-					foreach (Configuration config in cfgList)
+					foreach (Configuration config in Configuration.LoadSessions())
 					{
 						if (config.Id.ToString() == item.Name)
 						{
@@ -618,19 +673,14 @@ namespace GnomeRDP
 			this.RefreshSessionList();
 		}
 		
-		private void OnTrayClicked(object sender, ButtonPressEventArgs a)
+		private void OnUseKeyringClicked(object sender, EventArgs a)
 		{
-			switch (a.Event.Button)
-			{
-				case 1:
-					OnTrayIconClicked(sender, a);
-					break;
-				case 3:
-					OnTrayIconPopup(sender, a);
-					break;
-			}
+			bool value = !Options.UseKeyring;
+			
+			((Gtk.CheckMenuItem)sender).Active = value;
+			Options.UseKeyring = value;
 		}
-		
+						
 		private void OnTrayIconClicked(object sender, EventArgs a)
 		{
 			if (wndMain.Visible)
@@ -654,12 +704,8 @@ namespace GnomeRDP
 		
 		private Menu CreateSessionsPopupMenu(int parentId, Menu parentMenu)
 		{
-			ArrayList cfgList;
-			Configuration cf = new Configuration();
-			cfgList = cf.LoadSessions();
-			
 			Menu submenu = new Menu();
-			foreach (Configuration cfg in cfgList)
+			foreach (Configuration cfg in Configuration.LoadSessions())
 			{
 				if (cfg.ParentId == parentId)
 				{
